/**************************************************************************
      kooldock.cpp  -  description
      -------------------
      begin                : Tue Jun 10 22:18:34 BST 2003
      copyright            : (C) 2003 by KoolDock team
    email                : 
***************************************************************************/

/***************************************************************************
      *                                                                         *
      *   This program is free software; you can redistribute it and/or modify  *
      *   it under the terms of the GNU General Public License as published by  *
      *   the Free Software Foundation; either version 2 of the License, or     *
      *   (at your option) any later version.                                   *
      *                                                                         *
***************************************************************************/

#include <stdlib.h>
#include <math.h>

#include <qpixmap.h>
#include <qimage.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qtooltip.h>
#include <qrect.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qbrush.h>
#include <qpoint.h>
#include <qfont.h>
#include <qdatetime.h>

#include <kwin.h>
#include <kprocess.h>
#include <netwm_def.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kcmdlineargs.h>
#include <krun.h>
#include <kurifilter.h>
#include <kpixmapio.h>
#include <kstandarddirs.h>
#include <kaboutapplication.h>
#include <qlistbox.h>
#include <keditlistbox.h>

// for "renicing" the bar
#include <sys/time.h>
#include <sys/resource.h>

#include "kooldock.h"

#include <kconfig.h> 
#include <kdebug.h>
#include "setupdialogprg.h"

// macros

#define min(a, b) (a < b) ? (a) : (b)
#define max(a, b) (a > b) ? (a) : (b)

// compiler issues
#if __GNUC__ == 2 && __GNUC_MINOR__ <= 95
#  define _GCC295
#endif
#include <dcopclient.h>
#include <iostream>
#include <qdatastream.h>

      KoolDock::KoolDock(QWidget *parent, const char *name) : QWidget(parent, name, WStyle_Customize | WStyle_NoBorder | WNoAutoErase | WDestructiveClose | WMouseNoMask | WStyle_StaysOnTop | WX11BypassWM)
{
  // Default configuration paths
  progPath = locateLocal("data", "kooldock");
  menuPath = progPath + QString("/menu/");
  confFile = locateLocal("config", "kooldockrc");

  // Load the configuration
  loadConf();
  // Lets initialize the systray counter
  numSystray=0;
  
  // first, we set our "nice" priority
  if (fPriority > 19) fPriority = DEF_PRIORITY;
  if (fPriority < 0) fPriority = DEF_PRIORITY;
  setpriority(0, 0, fPriority);
  debug(QString("Priority: %1").arg(fPriority));

  iwBig2 = iwBig;

  iDist = iwSmall + iSpace;
  funcW = (int)(iDist*fAmount/2);
  funcH = iwBig - iwSmall;
  iy = iSpace/ 2;

  info = new NETWinInfo( qt_xdisplay(), winId(), qt_xrootwin(), NET::WMState);
  info->setDesktop( NETWinInfo::OnAllDesktops );
  info->setState(NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager, NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager);
  setBackgroundMode(NoBackground);

  KWin::setOnAllDesktops(winId(), true);
  KWin::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
  KWin::setType(winId(), NET::Dock);
  
  //KWin::setType(winId(), NET::Dock);

  items.setAutoDelete(TRUE);

  if(fShowKMenu==1)
  {
    items.append(new Item("kmenu", "dcop kicker kicker popupKMenu 0", "KMenu", iwSmall, iwBig2));
  }
  
  //Lets load the menu entries
  loadMenu();

  wm = new KWinModule();
  
  setupdlg = new SetupDialogPrg();	// create the setup dialog object.

  // Load the ignore List
  loadIgnore();

  //Create desktop clipper widget
  clipw = new clip();

  // Create the "xosd" widget
  xosdw = new xosd();
  xosd_st = 0;
  xosdw->setFont(xosdFont);
  if (xosdItalic == true) xosdw->setItalic();
  if (xosdBold == true) xosdw->setBold();
  xosdw->setSize(xosdSize);
  xosdw->setShadowOffset(xosdShadowOffset);
  xosdw->setColor(xosdColor);
  xosdw->setShadowColor(xosdShadowColor);
  
  // Show task bar
  if (fShowTaskbar)
  {
    firstTime=true;
      //Lets add the necesary windows
    addWindows();
    connect(wm, SIGNAL(windowAdded(WId)), SLOT(windowAdded(WId)));
    connect(wm, SIGNAL(windowRemoved(WId)), SLOT(windowRemoved(WId)));
    connect(wm, SIGNAL(windowChanged(WId, unsigned int)), SLOT(windowChanged(WId, unsigned int)));
    connect(wm, SIGNAL(activeWindowChanged(WId)), SLOT(activeWindowChanged(WId)));

    /* System Tray implementation (hope this works - Francisco) */
    connect(wm, SIGNAL(systemTrayWindowAdded(WId)), SLOT(systemTrayWindowAdded(WId)));
    connect(wm, SIGNAL(systemTrayWindowRemoved(WId)), SLOT(systemTrayWindowRemoved(WId)));
  }
  connect(wm, SIGNAL(currentDesktopChanged(int)), SLOT(currentDesktopChanged(int)));

  // Pseudo-Transparency code
  rootpix = new KRootPixmap(this);
  rootpix->setCustomPainting(TRUE);
  connect(rootpix, SIGNAL(backgroundUpdated (const QPixmap &)), SLOT(updateBackground(const QPixmap &)));
  rootpix->start();

  debug(QString("Items loaded: %1").arg(items.count()));

  //Simple work around to support xinerama (user defined)
  if(xinerama==0)
  {
    dw=QApplication::desktop()->width();
    dh = QApplication::desktop()->height();
  }
  else
  {
    dw=leftRes;
    dh=XinDesiredHeight;
  }
  rdh=QApplication::desktop()->height();
        
  // All icons to small size
  unsigned int i;
  for (i=0; i < items.count() + 1; i++)
  {
    iSize[i] = iwSmall;
  }
  for (i=0; i < items.count() + 1; i++)
  {
    cx[i] = iSpace + i*iDist + iwSmall/2;
    cur_cx[i] = cx[i];
  }
  if ((fOrientation & 1)==1) 
  {
    int tmpdw;
    tmpdw=dw;
    dw=dh;
    dh=tmpdw;
  }
  if (fOrientation==2 || fOrientation==1) 
  {
    QString tmpstr;
    tmpstr=fLeftImg;
    fLeftImg=fRightImg;
    fRightImg=tmpstr;
  }
        
  Offset=0;
  
  xosdw->move(0,rdh);
  xosdw->show();
  //setMouseTracking(TRUE);
  fExpanded=FALSE;
  if(fHidden==1)
  {
      // Timer to track the mouse position
      // When the mouse pointer reaches the bottom of the screen, the bar should popup in a userdefined time (int hideTimer)
      
    track2active=FALSE;
    trackTimer = new QTimer(this);
    connect(trackTimer, SIGNAL(timeout()), this, SLOT(trackTimerEnd()));
    trackTimer->start(15, FALSE);
  }
  
  ii_first = 0;
  ii_last = items.count() - 1;

  iOnClick = -1;
  onChangeTimer = NULL;
  Ybase = 0;
  soffset=0;
  last_ncx=0;

  /* For the the onscreen text (icon names) */
  nom = "";
  aux = "";

  showTimer = new QTimer(this);
  connect(showTimer, SIGNAL(timeout()), this, SLOT(showTimerTicked()));

  mTimer = new QTimer(this);
  connect(mTimer, SIGNAL(timeout()), this, SLOT(mTimerEnd()));
  
  // this timer is to check if the dock should be restarted or not
  // calling chkRestart() to check.
  resTimer = new QTimer(this);
  connect(resTimer, SIGNAL(timeout()), this, SLOT(chkRestart()));
  resTimer->start(15, FALSE);
  
  _filterData = new KURIFilterData();

  enterTimer = new QTimer(this);
  connect( enterTimer, SIGNAL(timeout()), this, SLOT(enterTimerTicked()) );

  mkbigTimer = new QTimer(this);
  connect(mkbigTimer, SIGNAL(timeout()), this, SLOT(mkbigTimerDo()));

  mksmallTimer = new QTimer(this);
  connect(mksmallTimer, SIGNAL(timeout()), this, SLOT(mksmallTimerDo()));

  waitTimer = new QTimer(this);
  connect(waitTimer, SIGNAL(timeout()), this, SLOT(waitTimerEnd()));
  QWMatrix m;
  m.rotate( -90*fOrientation );
  if (!fLeftImg.isEmpty()) 
  {
    LeftImg.load(fLeftImg);
    LeftImg = LeftImg.xForm( m );
  }
  if (!fRightImg.isEmpty()) 
  {
    RightImg.load(fRightImg);
    RightImg = RightImg.xForm( m );
  }
  if (!fCenterImg.isEmpty()) 
  {
    CenterImg.load(fCenterImg);
    CenterImg = CenterImg.xForm( m );
  }
  
  // repaint timer
  //doRepaint = TRUE;
  repaintTimer = new QTimer(this);
  connect(repaintTimer, SIGNAL(timeout()), this, SLOT(repaintSlot()));
  repaintTimer->start(fRepaintInterval, FALSE);
  
  QString welcomeMsg;
  if (fFirstRun == TRUE)
  {
  // Show welcome dialog.
    welcomeMsg =  QString("<center><B>Welcome</B></center>It appears to be the first time you run KoolDock.<br>"
        "By default the dock is hidden at the botton of the screen, move your mouse"
        " to the bottom edge of the screen and the dock will appear.<br>"
        "Right click on the dock for a menu that allows you to configure it<br>"
        "or run with <I>-o</I> parameter to have preferences window at start.<br>"
        "We hope this software is usefull for you<br>-- The KoolDock Team");
    KMessageBox::information(0L, i18n(welcomeMsg));
    saveConf();	// create initial configuration.
  }
  if (fHidden==1)
  {
    move (0,rdh);
  }
  debug("Kooldock has started");

  doUpdateGeometry();
  if ((fOrientation & 1)==1) {offscr = QPixmap(h1, w1); bottomBgf = QPixmap(h1, dw);} else {offscr = QPixmap(w1, h1); bottomBgf = QPixmap(dw, h1);}
  
  soffset = iwSmall/2 + func2(abs(1-funcW)) + func2(abs(iDist+1-funcW)) + func2(abs(2*iDist+1-funcW)) + func2(abs(3*iDist +1-funcW))/2 - ((int)((iwBig - iwSmall)*0.6*fAmount) + iSpace*2)/2 - 4*iwSmall - (int)((3*iDist+1-funcW)*(func(0) + func(iDist) - 2*iwSmall)/(2*iDist))+1;
  
  //KMessageBox::information(0L, QString("%1").arg(args));
  
  
  animValue = 0;
  animState = TRUE;
  show();
  if (fHidden==0) 
  {
    mTimer->start(25, FALSE);
    resize2(w0,h0);
    paintEvent(NULL);
  }
  //clipping desktop
  clipw->resize(0,0);
  clipw->move(-10,-10);
  clipw->show();
  if (fHidden==0 && fClipping==1)
  {
    if (fOrientation==0) {clipw->clipDesktop(0, 0, 0, 0, 0, 0, 0, 0, 0, h0, 0, dh);setDesktopIconsArea(0,0,dw,dh-h0);}
    if (fOrientation==1) {clipw->clipDesktop(0, 0, 0, h0, 0, dh, 0, 0, 0, 0, 0, 0);setDesktopIconsArea(0,0,dh-h0,dw);}
    if (fOrientation==2) {clipw->clipDesktop(0, 0, 0, 0, 0, 0, h0, 0, dh, 0, 0, 0);setDesktopIconsArea(0,h0,dw,dh);}
    if (fOrientation==3) {clipw->clipDesktop(h0, 0, dh, 0, 0, 0, 0, 0, 0, 0, 0, 0);setDesktopIconsArea(h0,0,dh,dw);}
  }
}
KoolDock::~KoolDock()
{
  if (info) delete(info);
  if (wm) delete(wm);
  if (setupdlg) delete(setupdlg);
  if (rootpix) delete(rootpix);
  if (trackTimer) delete(trackTimer);
  if (showTimer) delete(showTimer);
  if (resTimer) delete(resTimer);
  if (waitTimer) delete(waitTimer);
  if (enterTimer) delete(enterTimer);
  if (mkbigTimer) delete(mkbigTimer);
  if (mksmallTimer) delete(mksmallTimer);
  if (onChangeTimer) delete(onChangeTimer);
  if (_filterData) delete(_filterData);
}

void KoolDock::repaintSlot()
{
        //kdDebug() << "repaintslot" << endl;
  if (doRepaint == TRUE)
  {
    update();
    doRepaint = FALSE;
  }
}

void KoolDock::mTimerEnd()
{
  QPoint pointer;
  pointer = QCursor::pos();
  lastXPos=pointer.x()-rx;
  lastYPos=pointer.y()-ry;

  if (fHidden==1)
  {
    if (pointer.y() >= pos().y() && pointer.y() <= (pos().y() + size().height()) && pointer.x() >= pos().x() && pointer.x() <= (pos().x() + size().width()))
    {
      mMoveEvent(pointer.x()-rx,pointer.y()-ry);
    }
    else
    {
      debug(QString("mTimer1Leave: %1 %2 %3 %4 %5 %6").arg(pointer.x()).arg(pointer.y()).arg(dh-h).arg(ry).arg(rx).arg(x+w));
      onleaveEvent(NULL);
    }
  }
  else
  {
    if (pointer.y() >= pos().y() && pointer.y() <= (pos().y() + size().height()) && pointer.x() >= pos().x() && pointer.x() <= (pos().x() + size().width()))
    {
      if (fExpanded==TRUE) 
      {if (mkbigTimer->isActive()==FALSE && mksmallTimer->isActive()==FALSE) 
      {
        mMoveEvent(pointer.x()-rx,pointer.y()-ry);
      }
      }
      else
      {if (mkbigTimer->isActive()==FALSE && mksmallTimer->isActive()==FALSE) 
      {
        debug(QString("mTimer2EnterEv: %1 %2").arg(pointer.x()-rx).arg(pointer.y()-ry));
        enterEvent(NULL);
      }
      }
    }
    else
    {
      if (fExpanded==TRUE) 
      {if (mkbigTimer->isActive()==FALSE && mksmallTimer->isActive()==FALSE) 
      {
        debug(QString("mTimer2Leave"));
        onleaveEvent(NULL);
      }
      }
    }
  }
  if (fSteppy==1) if (!mkbigTimer->isActive() || !mksmallTimer->isActive()) mMoveEvent(pointer.x()-rx, pointer.y()-ry);
}


void KoolDock::trackTimerEnd()
{
  QPoint pointer;
  pointer = QCursor::pos();
  int px, py;
  if ((fOrientation & 1) == 1)
  {
    px = pointer.y();
    py = pointer.x();
  }
  else
  {
    py = pointer.y();
    px = pointer.x();
  }
  if ((fOrientation & 2) == 2) py = dh-py;
  if (py+2 >= dh && px > x0 && px < x0+w0)
  {
    if(track2active==FALSE) QTimer::singleShot( hideTimer, this, SLOT(trackTimer2End()) );
    track2active=true;
  }
}

void KoolDock::trackTimer2End()
{
  QPoint pointer;
  pointer = QCursor::pos();
  int px, py;
  if ((fOrientation & 1) == 1)
  {
    px = pointer.y();
    py = pointer.x();
  }
  else
  {
    py = pointer.y();
    px = pointer.x();
  }
  if ((fOrientation & 2) == 2) py = dh-py;
  if (py+2 >= dh && px > x0 && px < x0+w0)
  {
    if(mksmallTimer->isActive()==TRUE) mksmallTimerstop();
    move( 0, rdh);
    raise();
    trackTimer->stop();
    getBottomBG();
    if(h1 > h0) getTopBG();
    debug(QString("tracktimer: grabWindow(%1, %2, %3, %4)").arg(x1).arg(dh - h1).arg(w1).arg(h1));
    move2( x, dh - h);
    resize2(w, h);
    Ybase = iwBig2;
    showTimer->start(1, FALSE);
    enterEvent(NULL);
    enterLocked=TRUE;
  }
  track2active=FALSE;
}
void KoolDock::showTimerTicked()
{
  Ybase = iwBig2*(fzoomSpeed-zoomStep)/fzoomSpeed;

  if (Ybase <= 0)
  {
    Ybase = 0;
    showTimer->stop();
  }
  doRepaint = TRUE;
}

void KoolDock::paintEvent(QPaintEvent *)
{
        
  // Draw backgound, icons, and all that stuff over the widget.
  QPainter p(&offscr);
  
  int i;
  int ii;
  int tmpw;
  double s0;
  if (!fExpanded) s0=0;
  if (fExpanded) s0=1;
  if (mkbigTimer->isActive()) s0 = zoomStep/fzoomSpeed;
  if (mksmallTimer->isActive()) s0 = (fzoomSpeed-zoomStep)/fzoomSpeed;
  if (s0>1) s0=1;
  
  debug(QString("painting"));

  // draw background (new engine)
  int bgY;
  QImage TempBG;
  QPixmap TempScaledLeft;
  //QPixmap TempScaledCenter;
  QPixmap TempScaledRight;
  int p1,p2,p3,p4=0,p5=0,p6,p7,p8,p9,p10,pt;
  if (w<dw) Offset=0;
  if (fExpanded==FALSE)
  {
    bgY = 0;
  }
  else
  {
    bgY = h1 - h0 + Ybase;
  }
  int firstX;
  int lastX = cur_cx[items.count()-1] - iSize[items.count()-1]/2 + adjust;
  lastX = lastX + iSize[items.count()-1];
  if (ii_last == (int)items.count()-1 && fShowBorders) lastX -=soffset;
  
  if (ii_first == 0)
  {
    firstX = cur_cx[0] - iSize[0]/2 - iSpace/2 + adjust + soffset;
    tmpw = lastX+iSpace + 1 - soffset;
    if (last_mx>=0)
    {
      tmpw -= adjust;
    }
  }
  else
  {
    firstX = cur_cx[0] - iSize[0]/2 - iSpace/2 + SPACE_W;
    lastX +=soffset;
    tmpw = lastX+iSpace;
  }
  
  if ((h==h0 || iwSmall==iwBig) && (iwSmall!=iwBig2))
  { // minimised mode
    if(fOrientation<2 || uw==w1)
    {
    p1=x0-x1;
    if ((fOrientation & 2)==2) {p2=0;} else {p2=h1-h0;}
    p3=w;
    p4=h;
    if ((fOrientation & 1)==1)
    {
      pt=p1; p1=p2; p2=pt;
      pt=p3; p3=p4; p4=pt;
    }
    bitBlt(&offscr, 0, 0, &bottomBg,p1,p2,p3,p4);	 // put the background into offscr
    }
    else
    {
      bitBlt(&offscr, 0, 0, &bottomBg);
    }
    if (dockOpacity>0)
    {
      p1=firstX-Offset;
      p2=0;
      p3=tmpw;
      p4=h;
      p5=p1;
      p6=p2;
      if(fOrientation>1 && uw==w1) {p6+=x0-x1;}
      if ((fOrientation & 1)==1)
      {
        pt=p1; p1=p2; p2=pt;
        pt=p3; p3=p4; p4=pt;
      }
      bitBlt(&offscr, p1, p2, &bottomBgf ,p5, p6, p3, p4);
    }
  }
  else
  {	// maximized mode
    p1=0;
    p2=0;
    p3=dw;
    p4=h1 - h0;
    p5=0;
    p6=h1 - h0;
    p7=dw;
    p8=h0;
    if ((fOrientation & 2)==2)
    {
      p6=0;
      p2=h0;
    }
    if ((fOrientation & 1)==1)
    {
      pt=p1; p1=p2;  p2=pt;
      pt=p3; p3=p4;  p4=pt;
      pt=p5; p5=p6;  p6=pt;
      pt=p7; p7=p8;  p8=pt;
    }
    if((h1 - h0)>0) bitBlt(&offscr, p1, p2, &topBg, 0, 0, p3, p4);		// put the top background
    bitBlt(&offscr, p5, p6, &bottomBg, p5, p6, p7, p8);//put the rest of the background
    if (dockOpacity>0)
    {
      p1=firstX-Offset;
      if ((fOrientation & 2)==2) {p2=0;} else {p2=h1 - h0;}
      p3=tmpw;
      p4=h0;
      if ((fOrientation & 1)==1)
      {
        pt=p1; p1=p2;  p2=pt;
        pt=p3; p3=p4;  p4=pt;
      }
      bitBlt(&offscr, p1, p2, &bottomBgf, p1, p2, p3, p4);
    }
  }
  // end drawing background

  KPixmapIO pixio;

  // Left side image
  p7=0;
  if (!fLeftImg.isEmpty())
  {
    TempBG = pixio.convertToImage(LeftImg);
    if ((fOrientation & 1)==1) {p2=LeftImg.height();} else {p2=LeftImg.width();}
    p7=p2/2;
    p3=h0;
    if ((fOrientation & 1)==1) {pt=p2; p2=p3; p3=pt;}
    TempScaledLeft = QPixmap(TempBG.scale(p2, p3));
    p1=firstX-Offset-p7;
    if ((fOrientation & 2)==2) 
    {
      if (fExpanded) {p2=-Ybase;} else {p2=0;}
    }
    else
    {p2=bgY;}
    if ((fOrientation & 1)==1) {pt=p1; p1=p2; p2=pt;}
    bitBlt(&offscr, p1, p2, &TempScaledLeft);
  }

  //Center image
  if (!fCenterImg.isEmpty())
  {
    TempBG = pixio.convertToImage(CenterImg);
      // hack!! to optimize drawing speed, now it scales the background image only when it's
      // necessary, I'd prefeer to use a bit more of memory instead of loosing speed.
      // -- Matias
    if ((fOrientation & 1)==1) 
    {
      p1=TempScaledCenter.height();
      p4=RightImg.height();
      p5=LeftImg.height();
    } 
    else 
    {
      p1=TempScaledCenter.width();
      p4=RightImg.width();
      p5=LeftImg.width();
    }
      
    if (ii_first > 0 && p1 != (lastX - RightImg.width()) )
    {
      p2=lastX - p4;
      p3=h0;
      if ((fOrientation & 1)==1) {pt=p2; p2=p3; p3=pt;}
      TempScaledCenter = QPixmap(TempBG.scale( p2, p3));
    }
    else if (ii_first <= 0 && p1 != (tmpw - RightImg.width() - iSpace) )
    {
      p2=lastX - p4 - adjust + 1;
      p3=h0;
      if ((fOrientation & 1)==1) {pt=p2; p2=p3; p3=pt;}
      TempScaledCenter = QPixmap(TempBG.scale( p2, p3));
    }
    p1=firstX + p5 - Offset-p7;
    if ((fOrientation & 2)==2) 
    {
      if (fExpanded) {p2=-Ybase;} else {p2=0;}
    }
    else
    {p2=bgY;}
    if ((fOrientation & 1)==1) {pt=p1; p1=p2; p2=pt;}
    bitBlt(&offscr, p1, p2, &TempScaledCenter);
  }

  // Right side image
  if (!fRightImg.isEmpty())
  {
    if ((fOrientation & 1)==1) 
    {
      p6=TempScaledCenter.height();
    } 
    else 
    {
      p6=TempScaledCenter.width();
    }
    TempBG = pixio.convertToImage(RightImg);
    p2=p4;
    p3=h0;
    if ((fOrientation & 1)==1) {pt=p2; p2=p3; p3=pt;}
    TempScaledRight = QPixmap(TempBG.scale(p2, p3));
    p1=firstX + p5 + p6 - Offset-p7;
    if ((fOrientation & 2)==2) 
    {
      if (fExpanded) {p2=-Ybase;} else {p2=0;}
    }
    else
    {p2=bgY;}
    if ((fOrientation & 1)==1) {pt=p1; p1=p2; p2=pt;}
    bitBlt(&offscr, p1, p2, &TempScaledRight);
  }
  // End drawing the backgound

    
  // draw borders
  if (fShowBorders)
  {
    p1=firstX-Offset;
    p3=SPACE_W;
    p4=h0-1;
    p5=tmpw;
    p6=SPACE_W;
    p7=tmpw - SPACE_W+firstX-Offset-1;
    if (h == h0) {p2=0;}
    else {p2=h1 - h0;}
    p8=p2;
    p9=p1;
    p10=p2;
    if ((fOrientation & 2)==2)
    {
      p2=0;
      p8=0;
      p10=p4;
    }
    if ((fOrientation & 1)==1)
    {
      pt=p1; p1=p2;  p2=pt;
      pt=p3; p3=p4;  p4=pt;
      pt=p5; p5=p6;  p6=pt;
      pt=p7; p7=p8;  p8=pt;
      pt=p9; p9=p10; p10=pt;
    }
    p.fillRect (QRect(p1, p2, p3, p4), QBrush(borderColor));
    p.fillRect (QRect(p9, p10, p5, p6), QBrush(borderColor));
    p.fillRect (QRect(p7, p8, p3, p4), QBrush(borderColor));
  }
  // end drawing borders

  Item *item;
  // new icon drawing engine
  int Ypos;	// Y position
  int Xpos;	// X position
  KPixmap *currentIcon;		// current icon pixmap
  for (i = 0; i < (int)items.count(); i++)
  {
    item = items.at(i);
      // start setting the icon position
    if(i < ii_first)
    {
        // left side icons
      Xpos = cur_cx[i] - iwSmall/2 + SPACE_W - Offset;
    }
    else if(i >= ii_first && i <= ii_last)
    {
        // zoomed icons (center)
      Xpos = cur_cx[i] - iSize[i]/2 + adjust - Offset+soffset*s0;
    }
    else if(i > ii_last)
    {
        // right side icons
      Xpos = cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 2 - Offset+soffset*s0*2;
    } // end getting icon positions
    if ((fOrientation & 2)==2) 
    {Ypos = iy-Ybase;}
    else
    {Ypos = h - iy - iSize[i]+Ybase;}
      
      // now, get the icon image
    if (Xpos<dw && Xpos+iwBig2>0)
    {
      currentIcon = new KPixmap(*item->getIcon(iSize[i]));
      if (items.at(i)->wIcon==TRUE)
      {
        for (ii=0;ii<(int)witems.count();ii++)
        {
          if (items.at(i)->getId()==witems.at(ii)->getId()) 
          {
            currentIcon = new KPixmap(*witems.at(ii)->getIcon(iSize[i]));
          }
        }
      }
        
      // misc icon effects and animations
      if (iOnClick == i)
      {
        // highlight the clicked icon
        KPixmapEffect::fade(*currentIcon, ((float)50)*0.01, QColor("#FFFFFF"));
        QTimer::singleShot( 10*60*1, this, SLOT( unhighlight() ) );	// unhighlight the icon in 1 second
      }
        
      // notify animation
      if (fShowNotification && item->isAnimed())
      {
        KPixmapEffect::fade(*currentIcon, ((float) animValue )*0.01, QColor("#FFFFFF"));
      }

      //Lets apply an effect to the active window in TaskBar
      if(item->getId()==wm->activeWindow())
      {
        KPixmapEffect::fade(*currentIcon, ((float)60)*0.01, QColor("#FFFFFF"));
      }
        
      // End drawing icon effects
        
      if ((fOrientation & 1)==1) {ii=Xpos;Xpos=Ypos;Ypos=ii;}
      // Draw the icon on the widget in its respective position
      bitBlt(&offscr, Xpos, Ypos, currentIcon);
      //p.fillRect (QRect(Xpos, Ypos, iSize[i], 1), QBrush(borderColor));
      //p.fillRect (QRect(Xpos, Ypos, 1, iSize[i]), QBrush(borderColor));
      // Destroy the unused currentIcon pixmap.
      delete currentIcon;
    }
  } // Rnd drawing icons
  
  // Draw the seperator between launcher and taskbar
  if(fShowTaskbar && ((numLaunchers>0 && items.count()>numLaunchers) || (fShowKMenu==1 && items.count()>1)))
  {
    i=numLaunchers-1;
    if (i>=0 && i<items.count()-1)
    {
      if(i < ii_first){Xpos = cur_cx[i] - iwSmall/2 + SPACE_W;}
      else if(i >= ii_first && i <= ii_last){Xpos = cur_cx[i] - iSize[i]/2 + adjust+soffset;}
      else if(i > ii_last){Xpos = cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 3+soffset *2;}
      Xpos+=iSize[i];
      i++;
      if(i < ii_first){Xpos += cur_cx[i] - iwSmall/2 + SPACE_W;}
      else if(i >= ii_first && i <= ii_last){Xpos += cur_cx[i] - iSize[i]/2 + adjust+soffset;}
      else if(i > ii_last){Xpos += cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 3 +soffset *2;}
      p1=Xpos/2 - Offset;
      p3=SPACE_W;
      p4=h0;
      if (fExpanded==FALSE) 
      {
        p2=h;
        if ((fOrientation & 2)==2) p2=0;
      }
      else if ( (signed)items.count() > numLaunchers)
      {
        p2=h1 - h0 + Ybase;
        if ((fOrientation & 2)==2) p2=-Ybase;
      }
      if ((fOrientation &1)==1)
      {
        pt=p1; p1=p2;  p2=pt;
        pt=p3; p3=p4;  p4=pt;
      }
      QRect sepRect(p1, p2, p3, p4);
      p.fillRect (sepRect, QBrush(sepColor));
    }
  }
  if((fSystray==1) && (numSystray>0))
  {
    i=items.count()-numSystray-1;
    if (i>=0 && i<items.count()-1)
    {
      if(i < ii_first){Xpos = cur_cx[i] - iwSmall/2 + SPACE_W;}
      else if(i >= ii_first && i <= ii_last){Xpos = cur_cx[i] - iSize[i]/2 + adjust+soffset;}
      else if(i > ii_last){Xpos = cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 3+soffset *2;}
      Xpos+=iSize[i];
      i++;
      if(i < ii_first){Xpos += cur_cx[i] - iwSmall/2 + SPACE_W;}
      else if(i >= ii_first && i <= ii_last){Xpos += cur_cx[i] - iSize[i]/2 + adjust+soffset;}
      else if(i > ii_last){Xpos += cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 3 +soffset *2;}
      p1=Xpos/2 - Offset;
      p3=SPACE_W;
      p4=h0;
      if(h == h0)
      {
        p2=h;
        if ((fOrientation & 2)==2) p2=0;
      }
      else
      {
        p2=h1 - h0 + Ybase;
        if ((fOrientation & 2)==2) p2=-Ybase;
      }
      if ((fOrientation &1)==1)
      {
        pt=p1; p1=p2;  p2=pt;
        pt=p3; p3=p4;  p4=pt;
      }
      QRect sepRect(p1, p2, p3, p4);
      p.fillRect (sepRect, QBrush(sepColor));
    }
  }
  
  // Finally update the widget
  if ((fOrientation & 1)==1)
  {bitBlt(this, 0, 0, &offscr, 0, 0, h, w, Qt::CopyROP);}
  else
  {bitBlt(this, 0, 0, &offscr, 0, 0, w, h, Qt::CopyROP);}
  

  //The following code prevents from casting shadow to kooldock's window by beryl
  //I don't like this solution, but I haven't any ideas how to do it better.
  QBitmap bm(size());
  QPainter mp;
  
  mp.begin(&bm, this);
  mp.fillRect(rect(), Qt::white);
  mp.setPen( Qt::black );
  mp.drawPoint(0,0);
  mp.end();

  setMask(bm);

}


void KoolDock::unhighlight()
{
  iOnClick = -1;
  doRepaint = TRUE;;
}


void KoolDock::mousePressEvent( QMouseEvent *e)
{
  int i;
  int Xpos;
  
  int mx = e->x();
  int my = e->y();
  if ((fOrientation & 1)==1) 
  {
    i=mx;
    mx=my;
    my=i;
  }
  if ((fOrientation & 2)==2) 
  {my=h1-my;}
  mx=mx;
  ButtonState srcButton = e->button();


  if (srcButton == Qt::LeftButton)
  {
    mx = mx+Offset;

    for (i=0; i<items.count(); i++)
    {
      if(i < ii_first){Xpos = cur_cx[i] - iwSmall/2 + SPACE_W;}
      else if(i >= ii_first && i <= ii_last){Xpos = cur_cx[i] - iSize[i]/2 + adjust+soffset;}
      else if(i > ii_last){Xpos = cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 3+soffset *2;}
      if ((mx+iSpace/2)>Xpos && (mx-iSpace)<(Xpos+iSize[i])) break;
    }

    if ((my >= h1 - iy - iSize[i]) && (my <= h1 - iy))
    {
      if ((i >= 0) && (i < (int)items.count()))
      {
        items.at(i)->anim(FALSE);
        WId id = items.at(i)->getId();
        if (id == 0)
        {
              // if user clicked on a quick launcher, launch the app
              // acknowledge on click
          iOnClick = i;
          doRepaint = TRUE;

              // run the app
          debug(QString("clicked: %1").arg(items.at(i)->getName()));
          if (items.at(i)->getName() == "KMenu")
          {
                  // show K Menu
            run(items.at(i)->getCommand());
          }
          else
          {
                  // run the application with KDE feedback (loading icon)
                  // we need to run the .desktop file, since the command
                  // may be a url or another KIO stuff like 'system:/'
            run(items.at(i)->getFilename());
          }
        }
        else
        {
          if(i>=((int)items.count()-numSystray))
          {
                  //KWin::forceActiveWindow(id);
            KWin::WindowInfo info;
            info = KWin::windowInfo(id);
            QString name = info.name();

            KWin::deIconifyWindow(id);
            kdDebug(0) << "Systray: " << id << " Name: " << name << " Estado: " << info.state() << endl;
                  //QMouseEvent me(e->type(), mapTo( topLevelWidget(), e->pos() ), e->globalPos(), e->button(), e->state() );

                  //QApplication::sendEvent(, &me);
          }
          else
          {
                  // if user clicked on a running task's icon, activate it
            if (id!=CurrentFocus)
            {
                    //KWin::activateWindow(id);
              cId = id;
              activateApp();
            }
            else
            {
              cId = id;
              minApp();
                    //KWin::iconifyWindow(id, true);
              CurrentFocus = winId();
            }
          }
          if(fHidden == 1 && fHideOnClick == 1) onleaveEvent(NULL);	// it's configurable
        }
      }
    }
  }
  else if(srcButton == Qt::RightButton)
  {
      // Right button.
    mx = mx + x1 - x0 - iSpace;
    i = mx/iDist;

      //Menu definitions
    KPopupMenu *deskpopup=new KPopupMenu();
    KPopupMenu *appMenu=new KPopupMenu();
    KPopupMenu *godesk=new KPopupMenu;
    KPopupMenu *tasklist=new KPopupMenu;
    KPopupMenu *popup=new KPopupMenu;
      
      // Main KoolDock popup menu
    popup->clear();
    popup->insertTitle(i18n("Main Menu"));
    popup->insertItem(SmallIcon("configure"), i18n("Edit Quick Launch &Menu"), this, SLOT(edit()));
    popup->insertItem(SmallIcon("pencil"), i18n("Edit &Preferences"), this, SLOT(editPref()));
    popup->insertItem(SmallIcon("redo"), i18n("&Reload configuration"), this, SLOT(restart()));
    popup->insertSeparator();
    popup->insertItem(SmallIcon("about_kde"), i18n("&About"), this, SLOT(about()));
    popup->insertSeparator();
    popup->insertItem(SmallIcon("exit"), i18n("E&xit"), this, SLOT(endProg()));
      // End Main KoolDock Menu

    if(fShowNav==1)
    {
        //Menu to be appended to MainMenu
      godesk->clear();
      connect(godesk, SIGNAL(activated(int)), this, SLOT(goToDesktop(int)));
      for(int index=0; index<KWin::numberOfDesktops(); index++)
      {
        QString tmp=i18n("Desktop &%1").arg(index+1);
        int menustate=godesk->insertItem(SmallIcon("forward"), tmp, index+1);
        if(KWin::currentDesktop()==(index+1)) godesk->setItemEnabled(menustate, false);
      }
        //End godesk Menu
        
        //tasklist; menu to access every window throught navigation menu
      QValueList<WId>::ConstIterator it;
      KWin::WindowInfo tmpinfo;
      tasklist->clear();
      connect(tasklist, SIGNAL(activated(int)), this, SLOT(goToWindow(int)));

      const int SUPPORTED_WINDOW_TYPES = NET::NormalMask | NET::DesktopMask | NET::DockMask | NET::ToolbarMask | NET::MenuMask | NET::DialogMask | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask;

      int counter_tmp=0;
      for(int index=0; index<KWin::numberOfDesktops(); index++)
      {
        tasklist->insertTitle(i18n("Desktop %1").arg(index+1), counter_tmp);
        counter_tmp++;
        int tmp_pos=counter_tmp;
        for (it=wm->windows().begin(); it != wm->windows().end(); ++it)
        {
          tmpinfo=KWin::windowInfo(*it);
          if(tmpinfo.isOnDesktop(index+1)==true)
          {
            if(tmpinfo.windowType(SUPPORTED_WINDOW_TYPES) == NET::Normal || tmpinfo.windowType(SUPPORTED_WINDOW_TYPES) == NET::Override || tmpinfo.windowType(SUPPORTED_WINDOW_TYPES) == NET::Dialog || tmpinfo.windowType(SUPPORTED_WINDOW_TYPES) == NET::Unknown)
            {
              if ((tmpinfo.state() & NET::SkipTaskbar) == 0)
              {
                tasklist->insertItem(SmallIcon("forward"), i18n(tmpinfo.name()), *it, counter_tmp);
                counter_tmp++;
              }
            }
          }
        }
        if(tmp_pos==counter_tmp) tasklist->removeItem(tmp_pos-1);
      }

        //Append Navigation Menu (godesk+Time)
      QDateTime timeEntry=QDateTime::currentDateTime();
      QString tmp=i18n("Navigation: Desktop %1").arg(KWin::currentDesktop());
      popup->insertTitle(tmp);
      popup->insertItem(SmallIcon("forward"), i18n("&Go to Desktop"), godesk);
      popup->insertItem(SmallIcon("forward"), i18n("&Task List"), tasklist);
      popup->insertSeparator();
      popup->insertItem(SmallIcon("clock"), timeEntry.toString());
        //End append Navigation Menu
    }

    if ((my >= h1 - iy - iSize[i]) && (my <= h1 - iy))
    {
      if ((i >= 0) && (i < (int)items.count()))
      {
        WId id = items.at(i)->getId();
        if (id == 0)
        {
              // if user right clicked on a quick launch icon.
          if (items.at(i)->getCommand() != "")
          {
                  // Edit the .desktop file
            if (items.at(i)->getName() == "KMenu")
            {
                    // if user right clicked on the K menu, show the configuration menu.
              popup->exec(QCursor::pos());
            }
            else
            {
              appMenu->clear();
              appMenu->insertTitle(i18n("Item menu"));
              iFilename = items.at(i)->getFilename();
              appMenu->insertItem(SmallIcon("configure"), i18n("&Edit item"), this, SLOT(editItem()));
              appMenu->insertItem(SmallIcon("fileclose"), i18n("&Delete item"), this, SLOT(removeItem()));
              appMenu->insertSeparator();
              appMenu->insertItem(SmallIcon("forward"), "&KoolDock", popup);
              appMenu->exec(QCursor::pos());
            }
          }
        }
        else
        {
              // If user clicked on a running task icon, show the app menu.
              // Pop-up menu for right-clicking on a taskbar icon.
          cId = id;
          QString appTitle;
          KWin::WindowInfo winfo;
          winfo = KWin::windowInfo(cId);
          int menustate;

              //Begin Move to Desktop Menu
          deskpopup->clear();
          QString tmp=i18n("Current Desktop &%1").arg(KWin::currentDesktop());
          menustate=deskpopup->insertItem(SmallIcon("forward"), tmp, 0);
          if(winfo.isOnDesktop(KWin::currentDesktop())==true) deskpopup->setItemEnabled(menustate, false);
          deskpopup->insertSeparator();
          for(int i=0; i<KWin::numberOfDesktops(); i++)
          {
            if(winfo.isOnDesktop(KWin::currentDesktop())==true && (i+1)==KWin::currentDesktop())
            {
              continue;
            }
            else
            {
              QString tmpd=i18n("Desktop &%1").arg(i+1);
              if((i+1)!=KWin::currentDesktop())
              {
                        
                menustate=deskpopup->insertItem(SmallIcon("forward"), tmpd, i+1);
                if(winfo.isOnDesktop(i+1)==true)
                {
                  deskpopup->setItemEnabled(menustate, false);
                }
              }
            }
          }
          connect(deskpopup, SIGNAL(activated(int)), this, SLOT(sendToDesktop(int)));
              //End Move to Desktop Menu

          appMenu->clear();
          appTitle = winfo.visibleName();
          appTitle.truncate(30);
          appMenu->insertTitle(appTitle);
          menustate = appMenu->insertItem(i18n("Mi&nimize"), this, SLOT(minApp()));
              // Don't show the "Minimize" menu if it's minimized.
          if (winfo.mappingState() == NET::Iconic) appMenu->setItemEnabled(menustate, false);
          menustate = appMenu->insertItem(i18n("Ma&ximize"), this, SLOT(maxApp()));
              // Don't show the "Maximize" menu if it's maximized
          if (winfo.state() == NET::Max) appMenu->setItemEnabled(menustate, false);
              // But show it if it's maximized AND minimized.
          if (winfo.mappingState() == NET::Iconic) appMenu->setItemEnabled(menustate, true);
          menustate = appMenu->insertItem(i18n("&Restore"), this, SLOT(restApp()));
              // Don't show the "Restore" menu if it is restored.
          if (winfo.mappingState() != NET::Iconic && winfo.state() != NET::Max) appMenu->setItemEnabled(menustate, false);

          appMenu->insertItem(SmallIcon("forward"), i18n("&Move to Desktop"), deskpopup);
          appMenu->insertSeparator();
          appMenu->insertItem(SmallIcon("fileclose"), i18n("&Close"), this, SLOT(closeApp()));
          appMenu->insertSeparator();
          appMenu->insertItem(SmallIcon("forward"), "KoolDock", popup);
          appMenu->exec(QCursor::pos());
        }
      }
    }
    else
    {
        // Show the configuration popup menu if the user clicked on another place.
      popup->exec(QCursor::pos());
    }
    if(deskpopup) delete(deskpopup);
    if(appMenu) delete(appMenu);
    if(godesk) delete(godesk);
    if(tasklist) delete(tasklist);
    if(popup) delete(popup);
  }
}

void KoolDock::mMoveEvent( int ex, int ey)
{
  if (fExpanded==FALSE) return;
  unsigned int i;
  int mx, my;
  int dx;
  unsigned int k;
  int cur_cx_desk;
  int p1=0,p2=0;
  int Xpos;
  
  if ((fOrientation & 1)==1) 
  {
    mx=ex;
    ex=ey;
    ey=mx;
  }
  if ((fOrientation & 2)==2) 
  {ey=h1-ey;}
  
  mx = ex + x1 - x0;
  
  if (w1>dw)
  {
    Offset=(w1-dw)*mx/w0;
    ex=ex+Offset;
  }
  else
  {
    Offset=0;
  }

  if (fSteppy==1) last_ncx=mx=(int)(last_ncx*0.7+cx[(ex + x1 - x0 - iSpace)/iDist]*0.3);
      
  //for (i = 0; i < items.count() + 1; i++)
  if (abs(mx - last_mx) >= UPDATE_DIST && fExpanded==TRUE || mkbigTimer->isActive() || mksmallTimer->isActive())
  {
    last_mx = mx;
    ii_first = 0;
    ii_last = items.count() - 1;

    for (i = 0; i < items.count() + 1; i++)
    {
      dx = abs(cx[i] - mx);
      if (dx < funcW)
      {
        iSize[i] = func(dx);
      }
      else
      {
        iSize[i] = iwSmall;
        if (i < items.count())
        {
          if (cx[i] < mx)
          {
            ii_first = min(i + 1, items.count() - 1);
          }
          else if (ii_last == (int)items.count() - 1)
          {
            ii_last = max(i - 1, 0);
          }
        }
      }
    }
    cur_cx[0] = iSpace + iSize[0]/2;
      
    for(i=1; i < items.count() + 1; i++)
    {
      cur_cx[i] = cur_cx[i-1] + (iSize[i] + iSize[i-1])/2 + iSpace;
    }
    for(k=0; (k < items.count()-1) && (mx > cx[k]); k++);

    cur_cx_desk = ((int)((iwBig2 - iwSmall)*0.6*fAmount) + iSpace*2)/2 + cx[k] + (int)((cx[k] - mx)*(func(0) + func(iDist) - 2*iwSmall)/(2*iDist));
    adjust = cur_cx_desk - cur_cx[k];
    if (mkbigTimer->isActive()==FALSE && mksmallTimer->isActive()==FALSE)
    {
        if (mx < 0) onleaveEvent(NULL);
        if (mx > w0) onleaveEvent(NULL);
      if (ii_last < (int)items.count() - 1)
      {
        cur_cx[items.count() - 1] = w1 - iwSmall/2 - iSpace - adjust;
        for (i = items.count() - 2; (int)i >= ii_last + 1; i--)
        {
          cur_cx[i] = cur_cx[i+1] - iDist;
        }
        if (ii_last == 0) cur_cx[0] = cur_cx[1] - iDist - (iSize[0] - iSize[1])/2;
      }
    }
    else
    {
      ii_first = 0;
      ii_last = items.count() - 1;
    }
  }

  if (mx>w0 || mx<0)
  {
  ii_first = 0;
  ii_last = items.count() - 1;
  }
  if(showNames == 1 && iwBig == iwBig2)
  {
      // get the name of the item at the mouse position
    my = ey;
    mx = ex;

    for (i=0; i<items.count(); i++)
    {
      if(i < ii_first){Xpos = cur_cx[i] - iwSmall/2 + SPACE_W;}
      else if(i >= ii_first && i <= ii_last){Xpos = cur_cx[i] - iSize[i]/2 + adjust+soffset;}
      else if(i > ii_last){Xpos = cur_cx[i] - iwSmall/2 + adjust + SPACE_W - 3+soffset *2;}
      if ((mx+iSpace/2)>Xpos && (mx-iSpace)<(Xpos+iSize[i])) break;
    }
    QPoint pointer;
    pointer = QCursor::pos();
    mx = pointer.x()-x0-iSpace;
    if (i < MAX_ICONS+1)
    {
      if (((int)i >= 0) && (i < items.count()))
      {
        nom = items.at(i)->getName();
        //nom=QString("%1 %2 %2 %4").arg(adjust).arg(cur_cx_desk).arg(ey).arg(Offset);
      }
      else
      {nom = "";}
    }

    if(nom!=aux)
    {
      xosd_st=0;
    }
    if(xosd_st==0)
    {
      xosdw->setText(nom);
      xosd_st=1;
    }
    aux = nom;
    if (!mksmallTimer->isActive())
    {
      if (fOrientation==0)
      {
        p1=mx+x0 - (xosdw->w/2);
        p2=dh - h - xosdw->h;
      }
      if (fOrientation==1)
      {
        p1=dh-h1-xosdw->w;
        p2=ex- xosdw->h/2+ry;
      }
      if (fOrientation==2)
      {
        p1=mx+x0 - (xosdw->w/2);
        p2=h1;
      }
      if (fOrientation==3)
      {
        p1=h1;
        p2=ex- xosdw->h/2+ry;
      }
      if ((fOrientation & 1)==0)
      {
        if (p1<0) p1=0;
        if (p1+xosdw->w>dw) p1=dw-xosdw->w;
      }
      xosdw->move(p1, p2);
    }
  }
  doRepaint = TRUE;
  
}

//Mouse Wheel now switches desktops (away from user desktop+1, towards user desktop-1)
//-Francisco
void KoolDock::wheelEvent(QWheelEvent *e)
{
  if(e->delta()==120 && (KWin::currentDesktop()<KWin::numberOfDesktops()))
  {
    KWin::setCurrentDesktop(KWin::currentDesktop()+1);
  }
  if((e->delta()==-120) && (KWin::currentDesktop()>0))
  {
    KWin::setCurrentDesktop(KWin::currentDesktop()-1);
  }
}


void KoolDock::mkbigTimerDo()
{
  iwBig = iwBig2*zoomStep/fzoomSpeed+iwSmall*(fzoomSpeed-zoomStep)/fzoomSpeed;	// frames to grow up the dockbar
  if (iwBig<iwSmall) iwBig=iwSmall;
  zoomStep=zoomStep+5;
  
  if (fzoomSpeed<=zoomStep)
  {
    iwBig = iwBig2;
    mkbigTimer->stop();
    last_mx=-1;
    debug(QString("Stopped mkbigTimer"));
  }
  iy = (iDist - iwSmall) / 2;
  funcH = iwBig - iwSmall;
  mMoveEvent(QWidget::mapFromGlobal ( QCursor::pos() ).x(), QWidget::mapFromGlobal ( QCursor::pos() ).y());
  doRepaint = TRUE;
}

void KoolDock::mksmallTimerstop()
{
  int i;
  mksmallTimer->stop();
  xosdw->move(0,rdh);
  fExpanded=FALSE;
  debug(QString("Stopped mksmallTimer"));
  if (fHidden==0)
  {
    w = w0;
    x = x0;
    h = h0;
  }
      // normalize the dock
  for (i = 0; i < (int)items.count() + 1; i++)
  {
    iSize[i] = iwSmall;
    cur_cx[i] = cx[i];
  }
      
  ii_first = 0;
  ii_last = items.count() - 1;
  ii_first = 0;
  ii_last = items.count() - 1;
  adjust = 0;
  iwBig = iwBig2;
  if (fHidden == 0)
  {
    move2( x, dh - h );
    resize2(w, h);
  }
  else
  {
    move( 0, rdh);
    resize2(w, h);
  }
  //paintEvent(NULL);
  //doRepaint = TRUE;
}

void KoolDock::mksmallTimerDo()
{
  iwBig = iwBig2*(fzoomSpeed-zoomStep)/fzoomSpeed+iwSmall*zoomStep/fzoomSpeed;
  if (iwBig<iwSmall) iwBig=iwSmall;
  if (fzoomSpeed<=zoomStep)
  {
    mksmallTimerstop();
  }
  else
  {
    if (fHidden == 1) Ybase = iwBig2*zoomStep/fzoomSpeed;
    funcH = iwBig - iwSmall;
    iy = (iDist - iwSmall) / 2;
    mMoveEvent(lastXPos,lastYPos);
    doRepaint = TRUE;
  }
  zoomStep=zoomStep+5;
}

void KoolDock::waitTimerEnd()
{
  enterLocked = false;
  waitTimer->stop();
  debug("waitTimerEnd();");
  enterEvent(NULL);
}


void KoolDock::enterEvent(QEvent *)
{
  if (fExpanded==TRUE) return;
  if (enterLocked == true)
  {
    debug("enterEvent (enterLocked)");
    waitTimer->start(15, FALSE);
    return;
  }
  if (mksmallTimer->isActive()==TRUE) {if (fHidden==1) {return;} else {mksmallTimer->stop();}}

  w=w1;
  x=x1;
  h=h1;
  
  if (ry!=(dh-h1)) getTopBG();
  if (dockOpacity > 0)	{bitBlt(&bottomBgf,0,0,&bottomBg);KPixmapEffect::fade(bottomBgf, ((float)dockOpacity)*0.01, bgColor);}
  
  resize2(w,h);
  move2(x,dh-h);
  adjust=x0-x1;
  iwBig = iwSmall;
  funcH = iwBig - iwSmall;
  iy = (iDist - iwSmall) / 2;
  
  fExpanded=TRUE;
  enterLocked=TRUE;
  paintEvent(NULL);
  
  debug("enterEvent");
  //enterTimer->start( 1, TRUE );

  
  zoomStep=0;

  mkbigTimer->start(5, FALSE);
  if (fHidden==1) mTimer->start(25, FALSE);
}

void KoolDock::onleaveEvent(QEvent *)
{
  if (mksmallTimer->isActive()==TRUE) return;
  debug("onleaveEvent");
  lastXPos = QWidget::mapFromGlobal ( QCursor::pos() ).x();
  lastYPos = QWidget::mapFromGlobal ( QCursor::pos() ).y();
  if (fHidden==1) mTimer->stop();
  if (mkbigTimer->isActive())
  {
    zoomStep=fzoomSpeed-zoomStep;
    mkbigTimer->stop();
  }
  else
  {
    zoomStep=0;
  }
  mksmallTimer->start(5, FALSE);
  
  if(showNames==1)
  {
      //xosdw->hide();
    xosdw->move(0,rdh);
    xosd_st=0;
  }
  enterLocked = false;
  if(fHidden==1) 
  {
    trackTimer->start(15, FALSE);
  }
}


void KoolDock::edit()
{
  editPref();
  setupdlg->tabWidget->setCurrentPage(1);
}


void KoolDock::run(QString command)
{
  QString exec;

  kapp->propagateSessionManager();

  _filterData->setData( command.stripWhiteSpace() );
  QStringList filters;
  filters << "kurisearchfilter" << "kshorturifilter";
  KURIFilter::self()->filterURI( *(_filterData), filters );

  QString cmd = (_filterData->uri().isLocalFile() ? _filterData->uri().path():_filterData->uri().url());

  // Nothing interesting. Quit!
  if(cmd.isEmpty())
  {
    KMessageBox::sorry(0L, i18n("You have to enter a command to execute or a URL to be opened first."));
    return;
  }
  else
  { 
    switch( _filterData->uriType() )
    {
      case KURIFilterData::LOCAL_FILE:
      case KURIFilterData::LOCAL_DIR:
      case KURIFilterData::NET_PROTOCOL:
      case KURIFilterData::HELP:
      {
        (void) new KRun( _filterData->uri() );
        return;
      }
      case KURIFilterData::EXECUTABLE:
      case KURIFilterData::SHELL:
      {
        exec = cmd;
        if( _filterData->hasArgsAndOptions() ) cmd += _filterData->argsAndOptions();
        break;
      }
      case KURIFilterData::UNKNOWN:
      case KURIFilterData::ERROR:
      default:
        KMessageBox::sorry( 0, i18n("<qt>The program name or command <b>%1</b>\n"
            "cannot be found. Please correct the command\n"
                "or URL and try again</qt>").arg( cmd ) );
        return;
    }
  }
  
  if(KRun::runCommand( cmd, exec, "" ))
  {
    return;
  }
  else
  {
    KMessageBox::sorry( 0, i18n("<qt>Could not run <b>%1</b>.\nPlease correct"
        " the command or URL and try again.</qt>").arg( cmd ) );
    return;
  }
}


// Parabolic function
int KoolDock::func(int x)
{
  int res = iwBig - (int)((x*x*funcH)/(funcW*funcW));
  return res;
}
int KoolDock::func2(int x)
{
  int res;
  if (x<funcW)
  {res = iwBig - (int)((x*x*funcH)/(funcW*funcW));}
  else
  {res=iwSmall;}
  return res;
}

//Here we add windows to the taskbar
void KoolDock::addWindows()
{
  // Show task bar
  if(fShowTaskbar)
  {
    KWin::WindowInfo info;
    QValueList<WId>::ConstIterator it;

      //First we removed every taskbar item
    if(fCurrent==1)
    {
      int count=items.count()-numSystray;
      for(int i=count; i>=numLaunchers; i--)
      {
        items.remove(i);
      }
    }

      //Lets go through all the windows
    for (it=wm->windows().begin(); it != wm->windows().end(); ++it)
    {
      if(firstTime==true) addwTask(*it);
      if(fCurrent==1)
      {
        info=KWin::windowInfo(*it);
            //If it belongs to the current desktop, lets add it
        if(info.isOnDesktop(KWin::currentDesktop())==true)
        {
          addTask(*it);
        }
      }
      else
      {
        if(firstTime==true)
        {
          addTask(*it);
        }
      }
    }
    firstTime=false;
  }
}


//What happens when we change desktops
void KoolDock::currentDesktopChanged(int)
{
  funcH = iwBig2 - iwSmall;
  iwBig = iwBig2;
  addWindows();
  doUpdateGeometry();
  move(0,rdh);
  getTopBG();
  if (fExpanded) move2(x1,dh-h1);
  if (fHidden==0 && !fExpanded) move2(x0,dh-h0);
  xosdw->move(0,rdh);
}

void KoolDock::windowAdded(WId id)
{
  KWin::WindowInfo info;
  info = KWin::windowInfo(id);
  QString name = info.name();
  if (name == "kooldock xosd window")
  {
    debug("Not adding OSD window to taskbar.");
    return;
  }
  if (name == "kooldock")
  {
    debug("Not adding ourselves.");
    return;
  }
        
  debug(QString("windowAdded(%1)").arg(name));

  addwTask(id);
  addTask(id);
  doUpdateGeometry();
}

void KoolDock::windowRemoved(WId id)
{
  unsigned int i;
  Item *item;
  debug(QString("windowRemoved(%1)").arg(id));

  KWin::WindowInfo info;
  info = KWin::windowInfo(id);
  QString name = info.name();
        
  if (name == "kooldock xosd window")
  {
    debug("Not removing OSD window from taskbar.");
    return;
  }
  for (i=0; i < witems.count(); i++ )
  {
    item = witems.at(i);
    if(item->getId() == id)
    {
      witems.remove(i);
      break;
    }
  }
  for (i=numLaunchers; i < items.count()-numSystray; i++ )
  {
    item = items.at(i);
    if(item->getId() == id)
    {
      items.remove(i);
      break;
    }
  }
  doUpdateGeometry();
}


void KoolDock::windowChanged(WId id, unsigned int properties)
{
  Item *item;
  Item *witem;
  bool iconChanged, nameChanged;
  int i;
  int j;
  bool toRepaint = FALSE;

  KWin::WindowInfo winfo;
  winfo = KWin::windowInfo(id);
        
  iconChanged = (properties & NET::WMIcon);
  nameChanged = (properties & NET::WMName);
  properties = 0;
        
  if (id != winId())
  {
    for (i=numLaunchers, item = items.at(numLaunchers); item; i++, item = items.next())
    {
      if(item->getId() == id)
      {
        if (ignored(winfo.name()) == true) break;
        if (nameChanged)
        {
              // Window title changed.
          debug(QString("windowChanged (nameChanged)"));
              // FIXME: on gcc2.95 item->setName() falls on a loop and gets a segfault. (?)
#ifndef _GCC295
          item->setName(winfo.name());
#endif
              
          if ((fShowNotification) && (id != wm->activeWindow()))
          {
            debug(QString("windowChanged (inactive window changed)"));
                  // An inactive window's title changed
                  // notify user
            item->anim(TRUE);
            if(onChangeTimer == NULL)
            {
              onChangeTimer = new QTimer(this);
              connect( onChangeTimer, SIGNAL(timeout()), this, SLOT(onChangeTimerTicked()));
              onChangeTimer->start( ON_CHANGE_ANIM_INTERVAL, FALSE );
            }
            toRepaint = TRUE;
          }
        }
            
        if(iconChanged)
        {
          debug(QString("windowChanged (iconChanged)"));
              // Window icon changed.
          for (j=0, witem = witems.at(0); witem; j++, witem = witems.next())
          {
            if(witem->getId() == id) witem->setIcon(KWin::icon(id, iwBig2, iwBig2, TRUE));
          }
          toRepaint = TRUE;
        }
            
        if(toRepaint) doRepaint = TRUE;
        break;
      }
    }
  }
}

void KoolDock::systemTrayWindowAdded(WId id)
{
  if(fSystray==1)
  {
    KWin::WindowInfo info;
    info = KWin::windowInfo(id);
    QString name = info.name();
    QPixmap pix = KWin::icon(id, iwBig2, iwBig2, TRUE);

    numSystray++;
    if (ignored(info.name()) == false)
    {
      items.append(new Item(pix, id, name, iwSmall, iwBig2));
      debug(QString("addSystem Tray(%1)").arg(info.name()));
    }
    doUpdateGeometry();
  }
}

void KoolDock::systemTrayWindowRemoved(WId id)
{
  if(fSystray==1)
  {
    numSystray--;
    if(numSystray<0) numSystray=0;
    windowRemoved(id);
  }
}

void KoolDock::activeWindowChanged(WId id)
{
  if (id != winId()) CurrentFocus = id;
        
  // end notify animation
  unsigned int i;
  for (i=0; i<=items.count()-1; i++)
  {
    if (items.at(i)->getId() == id)
    {
      items.at(i)->anim(FALSE);
    }
  }
        
  bool toDelete = TRUE;
  for (i=0; i<=items.count()-1; i++)
  {
    if (items.at(i)->isAnimed()) toDelete = FALSE;
  }	
        
  if (toDelete && onChangeTimer != NULL)
  {
    delete(onChangeTimer);
    onChangeTimer = NULL;
    debug("Deleted onChangeTimer");
  }
  paintEvent(NULL);
}

void KoolDock::doUpdateGeometry()
{
  int dw1, dw0;
  iwBig=iwBig2;
  w0 = 2*iSpace + (items.count() - 1)*iDist + iwSmall;
  w = w0;
  w1 = w0 + (int)((iwBig - iwSmall)*0.6*fAmount) + iSpace*2;
  x1 = (dw - w1) * fpercentPos / 100;
  if (w1>dw) {x1=0;}
  if (w1>dw) {dw1=dw;} else {dw1=w1;}
  if (w0>dw) {dw0=dw;} else {dw0=w0;}
  x0 = (dw1 - dw0)/2 + x1;
  if(xinerama==1){x0=x0+XinPreviousWidth; x1=x1+XinPreviousWidth;}
  x = x0;
  h0 = 2*iy + iwSmall;
  h = h0;
  h1 = 2*iy + iwBig;
  
  adjust = 0;

  if(fHidden == 0)
  {
    resize2(w, h);
    move2( x, dh - h );
  }
        
  if ((fOrientation & 1)==1) {offscr = QPixmap(h1, w1);} else {offscr = QPixmap(w1, h1);}
        
  ii_first = 0;
  ii_last = items.count() - 1;
  ii_first = 0;
  ii_last = ii_last;

  if (fExpanded==TRUE)
  {
    w=w1;
    h=h1;
    x=x1;
    funcH = iwBig2 - iwSmall;
    iy = (iDist - iwSmall) / 2;
    if(fHidden==1) 
    {
      move(0,rdh);
      getBottomBG();
      getTopBG();
        //move(rx,ry);
    }
    resize2(w, h);
    move2( x, dh - h );
  }
  else if (fHidden==1)
  {
    move(0,rdh);
  }
  debug(QString("w1: %1").arg(w1));
  debug(QString("x1: %1").arg(x1));
  if (fExpanded) {last_mx=-10; mMoveEvent(lastXPos,lastYPos);}
  doRepaint = TRUE;
  xosdw->move(0,rdh);
  
}

void KoolDock::addTask(WId id)
{
  

  KWin::WindowInfo info;
  info = KWin::windowInfo(id);
  const int SUPPORTED_WINDOW_TYPES = NET::NormalMask | NET::DesktopMask | NET::DockMask | NET::ToolbarMask | NET::MenuMask | NET::DialogMask | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask;
  QString name = info.name();
  
  if (info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Normal || info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Override || info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Dialog || info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Unknown)
  {
    if ((info.state() & NET::SkipTaskbar) == 0)
    {
      if (ignored(info.name()) == false)
      {
        QPixmap pix;
        items.insert(items.count()-numSystray, new Item(pix, id, name, iwSmall, iwBig2,  TRUE));
        cx[items.count()-numSystray-1] = iSpace + (items.count()-numSystray-1)*iDist + iwSmall/2;
        cur_cx[items.count()-numSystray-1] = cur_cx[items.count()-numSystray-2] + (iwSmall + iSize[items.count()-numSystray-2])/2 + iSpace;
        iSize[items.count()-numSystray-1]=iwSmall;
        debug(QString("addTask(%1)").arg(info.name()));
      }
    }
  }
}

void KoolDock::addwTask(WId id)
{
  KWin::WindowInfo info;
  info = KWin::windowInfo(id);
  const int SUPPORTED_WINDOW_TYPES = NET::NormalMask | NET::DesktopMask | NET::DockMask | NET::ToolbarMask | NET::MenuMask | NET::DialogMask | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask;
  QString name = info.name();
  
  if (info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Normal || info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Override || info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Dialog || info.windowType(SUPPORTED_WINDOW_TYPES) == NET::Unknown)
  {
    if ((info.state() & NET::SkipTaskbar) == 0)
    {
      if (ignored(info.name()) == false)
      {
        QPixmap pix = KWin::icon(id, iwBig2, iwBig2, TRUE);
        witems.append(new Item(pix, id, name, iwSmall, iwBig2));
      }
    }
  }
}

void KoolDock::loadConf()
{
  KConfig *config = KGlobal::config();
  
  config->setGroup("kooldock");
  fShowTaskbar = config->readNumEntry("ShowTaskbar", 0);
  fShowNotification = config->readNumEntry("ShowNotification", 0);
  showNames = config->readNumEntry("showNames", 1);
  xosdFont = config->readEntry("xosdFont", "Tahoma");
  xosdColor = config->readEntry("xosdColor", "#f1f1f1");
  xosdShadowColor = config->readEntry("xosdShadowColor", "#000000");
  xosdSize = config->readNumEntry("xosdSize", 17);
  xosdShadowOffset = config->readNumEntry("xosdShadowOffset", 2);
  fShowBorders = config->readNumEntry("ShowBorders", 0);
  borderColor = QColor(config->readEntry("BorderColor", "#b1c4de"));
  iwSmall = config->readNumEntry("SmallIconSize", 32);
  iwBig = config->readNumEntry("BigIconSize", 90);
  fAmount = config->readNumEntry("BigIconAmount", 5);
  if (fAmount>10) fAmount=10;
  if (fAmount<4) fAmount=4;
  fShowKMenu = config->readNumEntry("ShowKMenu", 0);
  dockOpacity = config->readNumEntry("DockOpacity", 0);
  iSpace = config->readNumEntry("iSpace", 10);
  if (dockOpacity < 0)
  {
    dockOpacity = 0;
  }
  else if (dockOpacity > 100)
  {
    dockOpacity = 100;
  }
  bgColor = QColor(config->readEntry("BackgroundColor", "#ccccff"));
  sepColor = QColor(config->readEntry("SeparatorColor", "#000000"));
  fPriority = config->readNumEntry("Priority", 3);
  fClipping = config->readNumEntry("Clipping", 0);
  fClipIcons = config->readNumEntry("ClipIconArea", 0);
  fpercentPos = config->readNumEntry("percentPos", 50);
  fOrientation = config->readNumEntry("fOrientation", 0);
  if (fOrientation<0 || fOrientation>3) fOrientation=0;
  fzoomSpeed = config->readNumEntry("zoomSpeed", 20);
  if (fpercentPos<0) fpercentPos=0;
  if (fpercentPos>100) fpercentPos=100;
  fHidden = config->readNumEntry("Hidden", 1);
  fHighLightTime = config->readNumEntry("HighLightTime", 2);
  fHideOnClick = config->readNumEntry("HideOnClick", 0);
  fSteppy = config->readNumEntry("fSteppy", 0);
  fShowFrames = config->readNumEntry("ShowFrames", 30);
  fGrowFrames = config->readNumEntry("GrowFrames", 40);
  fLeftImg = config->readEntry("LeftImg");
  fRightImg = config->readEntry("RightImg");
  fCenterImg = config->readEntry("CenterImg");
  fFirstRun = config->readBoolEntry("FirstRun", TRUE);
  xinerama = config->readNumEntry("Xinerama", 0);
  leftRes=config->readNumEntry("leftRes", QApplication::desktop()->width());
  XinDesiredHeight=config->readNumEntry("XinDesiredHeight", QApplication::desktop()->height());
  XinPreviousWidth=config->readNumEntry("XinPreviousWidth", 0);
  hideTimer = config->readNumEntry("hideTimer", 125);
  Solid = config->readNumEntry("Solid", 1);
  fShowShot = config->readBoolEntry("ShowSShot", TRUE);
  fShowNav=config->readNumEntry("Navigator", 1);
  fSystray=config->readNumEntry("Systray", 0);
  fCurrent=config->readNumEntry("currentDesktop", 0);
  curTheme=config->readEntry("Theme");
  fRepaintInterval = config->readNumEntry("RepaintInterval", 5);

  if(config->readNumEntry("xosdBold", 1) == 0)
  {
    xosdBold = false;
  }
  else
  {
    xosdBold = true;
  }

  if(config->readNumEntry("xosdItalic", 1) == 0)
  {
    xosdItalic = false;
  }
  else
  {
    xosdItalic = true;
  }
        
  if (iwSmall > iwBig)
  {
    iwSmall = 32;
    iwBig = 90;
  }
  if (!fShowTaskbar)
  {
    fShowNotification = 0;
  }
  if (fHidden > 1) fHidden = 1;
}


void KoolDock::loadIgnore()
{
  QString line;

  // read the ignorelist into 'ignoreList'
  QFile in(QString(progPath + "/ignorelist.conf"));

  //clear de ignoreList and the KEditListBox (ignoreBox)
  ignoreList.clear();
  setupdlg->ignoreBox->clear();
  if (in.open(IO_ReadOnly))
  {
    while (in.readLine(line, MAX_LEN) != -1)
    {
        //append the item and strip the last char (new line)
      ignoreList.append(line.mid(0, line.length()-1));
    }
    in.close();
    if(!ignoreList.isEmpty()) setupdlg->ignoreBox->insertStringList(ignoreList);
  }
}


void KoolDock::saveConf()
{
  KConfig *config = KGlobal::config();
  
  config->setGroup("kooldock");
  config->writeEntry("ShowTaskbar", fShowTaskbar);
  config->writeEntry("ShowNotification", fShowNotification);
  config->writeEntry("ShowBorders", fShowBorders);
  config->writeEntry("BorderColor", borderColor.name());
  config->writeEntry("SmallIconSize", iwSmall);
  config->writeEntry("BigIconSize", iwBig2);
  config->writeEntry("BigIconAmount", fAmount);
  config->writeEntry("DockOpacity", dockOpacity);
  config->writeEntry("BackgroundColor", bgColor.name());
  config->writeEntry("SeparatorColor", sepColor.name());
  config->writeEntry("ShowKMenu", fShowKMenu);
  config->writeEntry("Priority", fPriority);
  config->writeEntry("Clipping", fClipping);
  config->writeEntry("ClipIconArea", fClipIcons);
  config->writeEntry("percentPos", fpercentPos);
  config->writeEntry("zoomSpeed", fzoomSpeed);
  config->writeEntry("Hidden", fHidden);
  config->writeEntry("fOrientation", fOrientation);
  config->writeEntry("ShowFrames", fShowFrames);
  config->writeEntry("GrowFrames", fGrowFrames);
  config->writeEntry("showNames", showNames);
  config->writeEntry("xosdFont", xosdFont);
  config->writeEntry("xosdColor", xosdColor);
  config->writeEntry("xosdShadowColor", xosdShadowColor);
  config->writeEntry("xosdSize", xosdSize);
  config->writeEntry("HideOnClick", fHideOnClick);
  config->writeEntry("fSteppy", fSteppy);
  config->writeEntry("HighLightTime", fHighLightTime);
  config->writeEntry("iSpace", iSpace);
  config->writeEntry("FirstRun", FALSE);
  config->writeEntry("Xinerama", xinerama);
  config->writeEntry("hideTimer", hideTimer);
  config->writeEntry("Solid", Solid);
  config->writeEntry("ShowSShot", fShowShot);
  config->writeEntry("Navigator", fShowNav);
  config->writeEntry("Systray", fSystray);
  config->writeEntry("currentDesktop", fCurrent);
  config->writeEntry("RepaintInterval", fRepaintInterval);
        
  if (xosdBold == true)
  {
    config->writeEntry("xosdBold", 1);
  }
  else
  {
    config->writeEntry("xosdBold", 0);
  }
  
  if (xosdItalic == true)
  {
    config->writeEntry("xosdItalic", 1);
  }
  else
  {
    config->writeEntry("xosdItalic", 0);
  }
        
  config->sync();
}


void KoolDock::editPref()
{
  bool tmp; //to save some code
        
  // open up the configuration dialog
  setupdlg->show();
  KWin::setOnDesktop(setupdlg->winId(), KWin::currentDesktop());
  
  // Fill dialog with values from configuration
  if (fHidden==1)
  {
    tmp=true;
  }
  else
  {
    tmp=false;
  }
  setupdlg->chk1->setChecked(tmp);
  setupdlg->hideTimer->setEnabled(tmp);
  setupdlg->hideOnClick->setEnabled(tmp);
  
  if(fShowTaskbar==1)
  {
    setupdlg->chk6->setEnabled(true);
    setupdlg->chkSshot->setEnabled(true);
    setupdlg->currentDesk->setEnabled(true);
  }
  else
  {
    setupdlg->chk6->setEnabled(false);
    setupdlg->chkSshot->setEnabled(false);
    setupdlg->currentDesk->setEnabled(false);
  }
  
  if(fSystray==1)
  {
    setupdlg->systray->setChecked(true);
  }
  else
  {
    setupdlg->systray->setChecked(false);
  }

  if(fCurrent==1)
  {
    setupdlg->currentDesk->setChecked(true);
  }
  else
  {
    setupdlg->currentDesk->setChecked(false);
  }
  //Even if start Hidden is disabled we load the stored value
  setupdlg->hideTimer->setValue(hideTimer);
  if (fHideOnClick == 1) setupdlg->hideOnClick->setChecked(TRUE);
  if (fSteppy == 1) setupdlg->Steppy->setChecked(TRUE);
  if (showNames==1)
  {
    tmp=true;
  }
  else
  {
    tmp=false;
  }
  setupdlg->chk2->setChecked(tmp);
  setupdlg->xosdBold->setEnabled(tmp);
  setupdlg->xosdItalic->setEnabled(tmp);
  setupdlg->xosdSize->setEnabled(tmp);
  setupdlg->spinBox2->setEnabled(tmp);
  setupdlg->kColorCombo1->setEnabled(tmp);
  setupdlg->kColorCombo_shadow->setEnabled(tmp);
  setupdlg->xosdFont->setEnabled(tmp);

  if(Solid==1)
  {
    setupdlg->chkSolid->setChecked(true);
    setupdlg->chkSolidSlot();
  }
  else
  {
    if(!curTheme.isEmpty()) setupdlg->theme->setCurrentText(curTheme);
    setupdlg->chkImg->setChecked(true);
    setupdlg->chkSolidSlot();
  }
  if (fShowBorders == 1) setupdlg->chk4->setChecked(TRUE);
  setupdlg->kColorCombo1->setColor(xosdColor);
  setupdlg->kColorCombo_shadow->setColor(xosdShadowColor);
  setupdlg->xosdFont->setCurrentFont(xosdFont);
  setupdlg->kColorCombo2->setColor(bgColor.name());
  setupdlg->kColorCombo3->setColor(borderColor.name());
  setupdlg->kColorCombo4->setColor(sepColor.name());
  setupdlg->spinBox1->setValue(dockOpacity);
  setupdlg->xosdSize->setValue(xosdSize);
  setupdlg->spinBox2->setValue(xosdShadowOffset);
  if (fShowTaskbar == 1) setupdlg->chk5->setChecked(TRUE);
  if (fShowNotification == 1) setupdlg->chk6->setChecked(TRUE);
  if (fShowKMenu == 1) setupdlg->chk7->setChecked(TRUE);
  setupdlg->iwSmall->setValue(iwSmall);
  setupdlg->iwBig->setValue(iwBig2);
  setupdlg->fAmount->setValue(fAmount);
  if (xosdBold == true) setupdlg->xosdBold->setChecked(TRUE);
  if (xosdItalic == true) setupdlg->xosdItalic->setChecked(TRUE);
  if (xinerama==1)
  {
    setupdlg->xrma->setChecked(TRUE);
    setupdlg->SleftRes->setEnabled(true);
    setupdlg->SHeight->setEnabled(true);
    setupdlg->PWidth->setEnabled(true);
    setupdlg->SleftRes->setValue(leftRes);
    setupdlg->SHeight->setValue(XinDesiredHeight);
    setupdlg->PWidth->setValue(XinPreviousWidth);
  }
  else
  {
    setupdlg->SleftRes->setEnabled(false);
    setupdlg->SHeight->setEnabled(false);
    setupdlg->PWidth->setEnabled(false);
    setupdlg->xrma->setChecked(false);
    setupdlg->SleftRes->setValue(QApplication::desktop()->width());
    setupdlg->SHeight->setValue(QApplication::desktop()->height());
  }
  if (fShowNav==1) setupdlg->nav->setChecked(true);
  setupdlg->chkSshot->setChecked(fShowShot);
  setupdlg->Priority->setValue(fPriority);
  if (fClipping==1) setupdlg->chClipping->setChecked(TRUE);
  if (fClipIcons==1) setupdlg->chClipIcons->setChecked(TRUE);
  setupdlg->zoomSpeed->setValue(fzoomSpeed);
  setupdlg->percentPos->setValue(fpercentPos);
  if (fOrientation==0) setupdlg->op0->toggle();
  if (fOrientation==1) setupdlg->op1->setChecked(true);
  if (fOrientation==2) setupdlg->op2->setChecked(true);
  if (fOrientation==3) setupdlg->op3->setChecked(true);
  loadIgnore(); //Load Ignore List

  setupdlg->tabWidget->setCurrentPage(0);
}


void KoolDock::loadMenu()
{
  QDir homeDir = QDir::home();
  if (homeDir.exists(menuPath))
  {
    QDir dir(menuPath);
      //First we make sure every index is correct!
      //Lets update the launchers index
    QString preffix;
    QStringList fileListRemove=dir.entryList("*.desktop");
    QStringList fileListRemove2=fileListRemove;
                    
      //Not really sure why I couldn't do this within the other for, something like  fileListRemove[i].remove(0,2).prepend(preffix), but hell it works
    for(unsigned int i=0; i<fileListRemove2.count(); i++)
    {
      fileListRemove2[i].remove(0,2);
    }
                    
      //Rename the files so they reflect their new position
    for(unsigned int i=0; i<fileListRemove.count(); i++)
    {
      if(i<10) preffix=QString("0%1").arg(i);
      if(i>=10) preffix=QString("%1").arg(i);
            
      dir.rename(fileListRemove[i], QString("%1%2").arg(preffix).arg(fileListRemove2[i]));
    }
      //End updating launchers index
      
    QStringList fileList = dir.entryList( "*.desktop");
    if(!fileList.isEmpty())
    {
      for ( QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it )
      {
        if ( ((*it) != ".") && ((*it) != "..") )
        {
          items.append(new Item(menuPath + (*it), iwSmall, iwBig2));
        }
      }
    }
  }
  else
  {
    debug("Config directory doesn't exits");
    QDir progPathDir(progPath);
    progPathDir.mkdir(progPath);
    QDir menuPathDir(menuPath);
    menuPathDir.mkdir(menuPath);
    debug("Created item directory");
        
      // create initial entries.
    addFile(QString("00_konqueror.desktop"), QString("konqueror"), QString("kfmclient openProfile webbrowsing"), QString("Konqueror Web Browser"), true, false, false, false, QString("na"));
    addFile(QString("01_home.desktop"), QString("kfm_home"), QString("kfmclient openProfile filemanagement"), QString("Home directory"), true, false, false, false, QString("na"));
    addFile(QString("02_kmail.desktop"), QString("kmail"), QString("kmail"), QString("KMail"), true, false, false, false, QString("na"));
    addFile(QString("03_kate.desktop"), QString("kate"), QString("kate"), QString("Kate"), true, false, false, false, QString("na"));
    addFile(QString("04_konsole.desktop"), QString("konsole"), QString("konsole"), QString("Konsole"), true, false, false, false, QString("na"));
    addFile(QString("05_kcontrol.desktop"), QString("kcontrol"), QString("kcontrol"), QString("Control Center"), true, false, false, false, QString("na"));
      
    loadMenu();
  }
  numLaunchers = items.count();
}


void KoolDock::about()
{
  KAboutApplication about(this, 0);
  about.exec();
}


void KoolDock::enterTimerTicked()
{
  if (enterTimer) enterTimer->stop();
}


void KoolDock::onChangeTimerTicked()
{
  if (animValue > 70) animState = FALSE;
  if (animValue < 5) animState = TRUE;
        
  if (animState == TRUE)
  {
    animValue = animValue + 5;
  }
  else
  {
    animValue = animValue - 5;
  }
  doRepaint = TRUE;
}


void KoolDock::updateBackground(const QPixmap& pix)
{
  if (fHidden == 0) bottomBg = QPixmap(pix);
  if (dockOpacity > 0)	{bitBlt(&bottomBgf,0,0,&bottomBg);KPixmapEffect::fade(bottomBgf, ((float)dockOpacity)*0.01, bgColor);}
  uw=w;
  doRepaint = TRUE;
  
}


void KoolDock::minApp()
{
  Item *item;
  int i;
  QPixmap shot;
  QImage tmpImg;
        
  if (fShowShot == TRUE)
  {
    for (i=0, item = witems.at(0); item; i++, item = witems.next())
    {
      if(item->getId() == cId)
      {
                                // take the screenshot
        shot = QPixmap::grabWindow(cId);
                                // scale the screenshot
        tmpImg = shot.convertToImage();
        tmpImg.scale(iwBig2, iwBig2);
        shot.convertFromImage(tmpImg);
                                // set the new item's icon
        item->setIcon(shot);
      }
    }
    doRepaint = TRUE;
  }
        
  KWin::iconifyWindow(cId, true);
}


void KoolDock::maxApp()
{
  Item *item;
  int i;
  if (fShowShot == TRUE)
  {
    for (i=0, item = witems.at(0); item; i++, item = witems.next())
    {
      if(item->getId() == cId)
      {
            // set default's item icon
        item->setIcon(KWin::icon(cId, iwBig2, iwBig2, TRUE));
      }
    }
  }
        
  KWin::setState(cId, NET::Max);
  KWin::activateWindow(cId);
}


void KoolDock::restApp()
{
  Item *item;
  int i;
  if (fShowShot == TRUE)
  {
    for (i=0, item = witems.at(0); item; i++, item = witems.next())
    {
      if(item->getId() == cId)
      {
            // set default's item icon
        item->setIcon(KWin::icon(cId, iwBig2, iwBig2, TRUE));
      }
    }
  }
  
  KWin::clearState(cId, NET::MaxVert + NET::MaxHoriz);
  KWin::deIconifyWindow(cId, true);
  KWin::activateWindow(cId);
}

void KoolDock::activateApp()
{
  Item *item;
  int i;
  if (fShowShot == TRUE)
  {
    for (i=0, item = witems.at(0); item; i++, item = witems.next())
    {
      if(item->getId() == cId)
      {
            // set default's item icon
        item->setIcon(KWin::icon(cId, iwBig2, iwBig2, TRUE));
      }
    }
  }
  KWin::activateWindow(cId);
  doRepaint = TRUE;
}

void KoolDock::closeApp()
{
  NETRootInfo app(qt_xdisplay(), NET::CloseWindow);
  app.closeWindowRequest(cId);
  debug(QString("Closed: %1").arg(cId));
}


void KoolDock::editItem()
{
  run(QString("kfmclient openProperties ") + iFilename);
}


void KoolDock::sendToDesktop(int desktop)
{
  if(desktop==0)
  {
    KWin::setOnDesktop(cId, KWin::currentDesktop());
  }
  else
  {
    KWin::setOnDesktop(cId, desktop);
  }
}


void KoolDock::goToDesktop(int desktop)
{
  KWin::setCurrentDesktop(desktop);
}


void KoolDock::goToWindow(int win)
{
  cId=(WId)win;
  activateApp();
}


void KoolDock::removeItem()
{
  QFile::remove(iFilename);
  restart();
}


void KoolDock::addFile(QString filename, QString iconname, QString execname, QString name, bool notify, bool terminal, bool tclose, bool cuser, QString puser)
{
  QString progPath = locateLocal("data", "kooldock");
  QString menuPath = progPath + QString("/menu/");
  QString confFile = locateLocal("config", "kooldockrc");

  QFile out(QString(menuPath + filename));
  if (out.open(IO_WriteOnly))
  {
    QTextStream out_s(&out);
    out_s << "[Desktop Entry]" << endl;
    out_s << "Comment=" << endl;
    out_s << "Encoding=UTF-8" << endl;
    out_s << "Exec=" << execname << endl;
    out_s << "GenericName=" << endl;
    out_s << "Icon=" << iconname << endl;
    out_s << "MimeType=" << endl;
    out_s << "Name=" << name << endl;
    out_s << "Path=" << endl;
    out_s << "ServiceTypes=" << endl;
    out_s << "SwallowExec=" << endl;
    out_s << "SwallowTitle=" << endl;
    out_s << "Terminal=false" << endl;
    out_s << "TerminalOptions=" << endl;
    out_s << "Type=Application" << endl;
    if(notify == true)
    {
      out_s << "X-KDE-StartupNotify=true" << endl;
    }
    else
    {
      out_s << "X-KDE-StartupNotify=false" << endl;
    }
    if(cuser==true)
    {
      out_s << "X-KDE-SubstituteUID=true" << endl;
      out_s << "X-KDE-Username=" << puser << endl;
    }
    else
    {
      out_s << "X-KDE-SubstituteUID=false" << endl;
      out_s << "X-KDE-Username=" << endl;
    }
    if(terminal==true)
    {
      out_s << "Terminal=true" << endl;
    }
    else
    {
      out_s << "Terminal=false" << endl;
    }
    if(tclose==true)
    {
      out_s << "TerminalOptions=\\s--noclose" << endl;
    }
    else
    {
      out_s << "TerminalOptions=" << endl;
    }
    out.close();
  }
}


bool KoolDock::ignored(QString appname)
{
  if(ignoreList.count()>0)
  {
    for(unsigned int i=0; i<ignoreList.count(); i++)
    {
      if(ignoreList[i]==appname)
        return(true);

    }
  }
  return(false);
}


#ifdef _ENABLE_DEBUG
void KoolDock::debug(QString message)
{
  // debugging function (show messages to stdout)
  kdDebug(0) << "DEBUG: " << message << endl;
}
#endif
#ifndef _ENABLE_DEBUG
void KoolDock::debug(QString)
{
}
#endif

void KoolDock::endProg()
{
if (fHidden==0 && fClipping==1)
{
  if ((fOrientation & 1)==1)
  {
  setDesktopIconsArea(0,0,dh,dw);
  }
  else
  {
  setDesktopIconsArea(0,0,dw,dh);
  }
}
  debug("Clean exit.");
  exit(0);
}

void KoolDock::restart()
{
  QString cmd;
  cmd = mainPath + " &";
  kdDebug(0) << "mainPath: " << mainPath << endl;
  debug("Restarting...");
  system(cmd.latin1());
  endProg();
}


void KoolDock::chkRestart()
{
  if (!setupdlg) return;
  // this function is called by a timer to check if the dock should be restarted
  // or not, depending on what was done in the configuration dialog.
  if (setupdlg->toRestart->isChecked())
  {
    setupdlg->toRestart->setChecked(FALSE);
    restart();
  }
}


void KoolDock::setMainPath(QString path)
{
  mainPath = path;
}

void KoolDock::setArgs(QString params)
{
  args = params;
  if (args=="-options" || args=="-o") editPref();
  
}

void KoolDock::move2(int nx, int ny)
{
  
  if ((fOrientation & 2) == 2) ny=0;
  if ((fOrientation & 1) == 1) 
  {rx=ny; ry=nx;}
  else
  {rx=nx; ry=ny;}
  if (rx==x0 && ry==dh-h0 && fExpanded==FALSE && w1>dw)
  {
    if (w0>dw)
    {
    Offset=(w1-dw)*lastXPos/dw - rx;
    move(0,ry);
    }
    else
    {
      Offset=0;
      move(rx,ry);
    }
  }
  else
  {
    move(rx,ry);
  }
}

void KoolDock::resize2(int nw, int nh)
{
  if (w1>dw && nw>w1) nw=dw;
  if ((fOrientation & 1) == 1) 
  {rw=nh; rh=nw;}
  else
  {rw=nw; rh=nh;}
  if (w1<dw) Offset=0;
  resize(rw,rh);
}
void KoolDock::getBottomBG()
{
  int p1,p2,p3,p4,pt;
  if (fHidden==0) return;
  p1=x1;
  if ((fOrientation & 2)==2) {p2=0;} else {p2=dh - h1;}
  p3=w1;
  p4=h1;
  if ((fOrientation & 1)==1)
  {
    pt=p1; p1=p2; p2=pt;
    pt=p3; p3=p4; p4=pt;
  }

  bottomBg = QPixmap::grabWindow(qt_xrootwin(), p1, p2, p3, p4);
  //bottomBg = QPixmap::grabWindow(qt_xrootwin(), x1, dh - h1, w1, h1);
}

void KoolDock::getTopBG()
{
  int p1,p2,p3,p4,pt;
  p1=x1;
  if ((fOrientation & 2)==2) {p2=h0;} else {p2=dh - h1;}
  p3=w1;
  p4=h1 - h0;
  if ((fOrientation & 1)==1)
  {
    pt=p1; p1=p2; p2=pt;
    pt=p3; p3=p4; p4=pt;
  }
  topBg = QPixmap::grabWindow(qt_xrootwin(), p1, p2, p3, p4);
  //topBg = QPixmap::grabWindow(qt_xrootwin(), x1, dh - h1, w1, h1 - h0);
}

void KoolDock::setDesktopIconsArea(int left, int top, int width, int height)
{
  if (fClipIcons==0) return;
  DCOPClient* client = KApplication::dcopClient();
  if (!client->isAttached()) {
    client->attach();
    client->registerAs("KoolDock");
  }

  for (int i = 0; i < KWin::numberOfDesktops(); i++) {
    QByteArray data;
    QDataStream args(data, IO_WriteOnly);
    args << left << top << width << height << i;
    client->send("kdesktop", "KDesktopIface", "desktopIconsAreaChanged(QRect, int)", data);
  }
}
